EXEC [dbo].[__AddStoredProcInfo]
    /* StoredProc_Name         */ 'ClusterNodes_CheckPrimary',
    /* Internal_Indicator      */ 'Y',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'Primary_Indicator',
    /* Description_Text        */ 'Checks if the specified node should be the primary node.'
GO

IF OBJECT_ID('[ClusterNodes_CheckPrimary]') IS NOT NULL DROP PROCEDURE [ClusterNodes_CheckPrimary]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ClusterNodes_CheckPrimary]
(
	@Node_Name NVARCHAR(100),
	@NodeType_Code CHAR(1),
	@FailOver_Minutes INT,
	@Primary_Indicator YNINDICATOR = NULL OUT
)
AS
BEGIN
SET NOCOUNT ON

--Default to 'N', do check for 'Y' later.
SET @Primary_Indicator = 'N'

BEGIN TRY
	BEGIN TRANSACTION

	DECLARE @Now DATETIME
	SET @Now = GETUTCDATE()

	DECLARE @Primary_Server_Name NVARCHAR(100),
	        @LastUpdated_Date DATETIME

	-- find the primary node (hopefully, just one) while locking the whole table
	SELECT TOP 1
	       @Primary_Server_Name = [Node_Name],
	       @LastUpdated_Date = [LastUpdated_Date]
	  FROM [ClusterNodes2] (TABLOCKX)
	 WHERE [Primary_Indicator] = 'Y'
	   AND [NodeType_Code] = @NodeType_Code

	-- no primaries or a stale primary, become the new primary
	IF @LastUpdated_Date IS NULL OR @LastUpdated_Date < DATEADD(MINUTE, @FailOver_Minutes * -1, @Now) BEGIN
		
		 -- in the strange, off chance there is no node, add it now
		IF NOT EXISTS(SELECT * FROM [ClusterNodes2] WHERE [Node_Name] = @Node_Name AND [NodeType_Code] = @NodeType_Code) BEGIN
			INSERT INTO [ClusterNodes2] ([Node_Name], [NodeType_Code], [LastUpdated_Date], [Primary_Indicator], [Live_Indicator])
			VALUES (@Node_Name, @NodeType_Code, @Now, 'Y', 'Y')
		END

		SET @Primary_Server_Name = @Node_Name
	END 

	-- set the primary as needed and update current server date
	 UPDATE [ClusterNodes2]
		SET [Primary_Indicator] = CASE WHEN [Node_Name] = @Primary_Server_Name THEN 'Y' ELSE 'N' END,
		    [LastUpdated_Date] = CASE WHEN [Node_Name] = @Node_Name THEN @Now ELSE [LastUpdated_Date] END
	  WHERE [NodeType_Code] = @NodeType_Code

	IF @Primary_Server_Name = @Node_Name BEGIN

		SET @Primary_Indicator = 'Y'

		-- if this is the primary node, look for stale nodes and mark Live_Indicator for them
		UPDATE [ClusterNodes2]
		   SET [Live_Indicator] = 'N'
		 WHERE [LastUpdated_Date] < DATEADD(MINUTE, @FailOver_Minutes * -1, @Now)

	END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [ClusterNodes_CheckPrimary] TO [ProGetUser_Role]
GO
