EXEC [__AddStoredProcInfo]
    /* StoredProc_Name         */ 'Users_CreateOrUpdateUser',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Creates or modifies a user.'
GO

IF OBJECT_ID('[Users_CreateOrUpdateUser]') IS NOT NULL DROP PROCEDURE [Users_CreateOrUpdateUser]
GO

SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO

CREATE PROCEDURE [Users_CreateOrUpdateUser]
(
	@User_Name USERNAME,
	@Display_Name NVARCHAR(100),
	@Email_Address VARCHAR(250) = NULL,
	@Groups_Xml XML
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF EXISTS(SELECT *
				FROM [Groups]
				WITH (UPDLOCK)
			   WHERE [Group_Name] = @User_Name)
		RAISERROR('Cannot create user "%s"; there is a group of the same name.', 16, 1, @User_Name)

	IF EXISTS(SELECT *
				FROM [Users]
				WITH (UPDLOCK)
			   WHERE [User_Name] = @User_Name)
	BEGIN

		UPDATE [Users]
		   SET [Display_Name] = @Display_Name,
			   [Email_Address] = @Email_Address
		 WHERE [User_Name] = @User_Name

		EXEC [Events_RaiseEvent] 'USRMDU', NULL, 'User_Name', @User_Name

	END
	ELSE
	BEGIN

		INSERT INTO [Users]
		(
			[User_Name],
			[Display_Name],
			[Email_Address]
		)
		VALUES
		(
			@User_Name,
			@Display_Name,
			@Email_Address
		)

		EXEC [Events_RaiseEvent] 'USRNWU', NULL, 'User_Name', @User_Name

	END

	DELETE [UserGroups]
	 WHERE [User_Name] = @User_Name

	INSERT INTO [UserGroups]
	(
		[User_Name],
		[Group_Name]
	)
	SELECT @User_Name,
		   GX.[value]('.', 'VARCHAR(50)') [Group_Name]
	  FROM @Groups_Xml.[nodes]('/groups/group') AS GroupsXml(GX)

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Users_CreateOrUpdateUser] TO [ProGetUser_Role]
GO